<?php

/**
 * @file
 * Functions for the 'link' link type for use with the Autolink module.
 */

/**
 * Save links from data sent from the node edit form.
 */
function autolink_ui_node_save_links($node, $links) {
  if (isset($links['keywords'])) {
    $path = 'node/'. $node->nid;

    $typed_keywords = $links['keywords'];
    $keyword_defaults = $links['defaults'];
    unset($links['keywords'], $links['defaults']);

    // The user changed the links: we need to delete the old keywords and replace them with new ones.
    $node_links = array();
    $result = db_query("SELECT * FROM {autolink_link} WHERE link_path = :path ORDER BY keyword ASC", array(':path' => $path));
    foreach ($result as $link) {
      $node_links[$link->group_type][$link->lid] = $link->keyword;
    }

    foreach ($typed_keywords as $group => $group_keywords) {
      if ($group_keywords !== $keyword_defaults[$group]) {
        if (!is_array($node_links[$group])) {
          $node_links[$group] = array();
        }
        // Delete link definitions that were removed from the keywords list.
        foreach ($node_links[$group] as $lid => $keyword) {
          if (preg_match('/\b'. preg_quote($keyword) .'\b/i', $group_keywords) == 0) {
            autolink_ui_link_delete($lid);
            drupal_set_message(t('Deleted link definition for keyword \'%keyword\'', array('%keyword' => $keyword)));
          }
        }

        // Finally populate the link definition array and save the new links.
        $keywords = drupal_explode_tags($group_keywords);
        foreach ($keywords as $keyword) {
          if (!autolink_ui_link_exists($keyword, $path)) {
            autolink_ui_node_save_link_by_nid($node->nid, $group, $keyword);
            drupal_set_message(t('Created new link definition for keyword \'%keyword\'', array('%keyword' => $keyword)));
          }
        }
      }
    }
  }
}

/**
 * Saves a new link definition based on the node ID of the node to which the link links.
 *
 * We populate all the link data based on the few parameters passed to the
 * function. This function is used during bulk operations, so we populate most
 * of the data with global and default settings that are established on the
 * Autolink settings form. We store the normal path in 'link_path' and get a
 * path alias if possible to store in the 'argument' field of the database.
 *
 * @param $nid
 *   A node ID.
 * @param $group
 *   A link group.
 * @param $keyword
 *   A keyword to store in the link definition.
 */
function autolink_ui_node_save_link_by_nid($nid, $group, $keyword) {
  // Populate the link definition data with settings and parameters.
  $link['type'] = 'node';
  $link['group_type'] = $group;
  $link['keyword'] = $keyword;
  $link['link_path'] = 'node/'. $nid;
  $link['operation'] = 'node_path';
  $link['case_sensitive'] = variable_get('autolink_case_sensitivity', 0);

  drupal_write_record('autolink', $link);
}

/**
 * Delete all links that point to a recently deleted node's path.
 */
function autolink_ui_delete_node_links($node) {
  // First get all the links that point to this node.
  $path = 'node/'. $node->nid;

  // We run a select query and delete link definitions individually to allow
  // other modules to respond to hook_autolink_link_delete().
  $links = array();
  $result = db_query("SELECT * FROM {autolink_link} WHERE link_path = :path", array(':path' => $path));
  foreach ($result as $link) {
    autolink_ui_link_delete($link->lid);
  }
}

/**
 * Callback for building link type specific form elements.
 */
function autolink_ui_node_method($edit, $type) {
  $form = array();
  switch ($type) {
    case 'title':
      // Get the node ID from the link_path to set default values.
      if (isset($edit['link_path']) && !empty($edit['link_path'])) {
        $args = explode('/', $edit['link_path']);
        $nid = $args[1];
      }

      $titles = autolink_ui_node_get_titles();

      $select['select'] = t('--Select a node--');
      $select += $titles;

      $form['node_title'] = array(
        '#type' => 'select',
        '#title' => t('Select a node title'),
        '#options' => $select,
        '#default_value' => isset($nid) ? $nid : NULL,
      );
      break;
    case 'path':
      $path = isset($edit['link_path']) ? autolink_ui_get_path($edit['link_path']) : '';
      $form['node_path'] = array(
        '#type' => 'textfield',
        '#title' => t('Enter a path'),
        '#required' => TRUE,
        '#default_value' => $path,
        '#description' => t('Provide a valid Drupal node path, normal or aliased.<br />' .
                            'Example: \'node/12\' or \'foo/bar\''),
      );
      break;
  }

  return $form;
}

/**
 * Validation callback for the node method form elements.
 */
function autolink_ui_node_method_validate($form_values, $type) {
  switch ($type) {
    case 'title':
      if ($form_values['node_title'] == 'select') {
        form_set_error('node_title', t('You must select a node.'));
      }
      break;
    case 'path':
      // Validate the path as an internal aliased or normal Drupal path.
      if (!drupal_valid_path($form_values['node_path']) && !drupal_lookup_path('source', $form_values['node_path']) && $form_values['node_path'] !== 'contact') {
        form_set_error('node_path', t('The path <i>%path</i> is invalid. You must provide a valid internal Drupal path.', array('%path' => $form_values['node_path'])));
      }
      break;
  }
}

 
/**
 * Implements hook_autolink_LINK_TYPE_alter().
 *
 * Buils a value for the link_path field.
 *
 * @param $link
 *   An array of form data for the link definition.
 */
function autolink_ui_autolink_node_alter(&$link, $method) {
  switch ($method) {
    case 'node_title':
      $link['link_path'] = 'node/'. $link['node_title'];
      break;
    case 'node_path':
      if ($source = drupal_lookup_path('source', $link['node_path'])) {
        $link['link_path'] = $source;
      }
      else {
        $link['link_path'] = $link['node_path'];
      }
      break;
  }
}

/**
 * Returns an array of node IDs and titles for the link add/edit form.
 */
function autolink_ui_node_get_titles() {
  $nodes = array();
  $result = db_query('SELECT n.nid, n.title FROM {node} n');
  foreach ($result as $node) {
    $nodes[$node->nid] = $node->title;
  }
  asort($nodes);
  return $nodes;
}
