<?php

/**
 * @file
 * Functions for the 'taxonomy' link type for use with the Autolink module.
 */

/**
 * Callback for building the taxonomy link form.
 */
function autolink_ui_taxonomy_method($edit, $type) {
  $form = array();

  switch ($type) {
    case 'select':
      if (isset($edit['link_path'])) {
        $args = explode('/', $edit['link_path']);
      }

      $terms = _autolink_ui_taxonomy_get_terms();

      $select['select'] = t('--Select a term--');
      $select += $terms;

      $form['term_select'] = array(
        '#type' => 'select',
        '#title' => t('Select a term'),
        '#required' => TRUE,
        '#options' => $select,
        '#default_value' => isset($args[2]) ? $args[2] : 'select',
      );
      break;
    case 'multiple':
      if (isset($edit['link_path']) && !empty($edit['link_path'])) {
        // Get the term names from tids by parsing the path.
        $args = explode('/', $edit['link_path']);
        $tids = explode(',', $args[2]);

        foreach ($tids as $tid) {
          $term = taxonomy_term_load($tid);
          $terms[$tid] = $term->name;
        }

        asort($terms);
        $terms_default = implode(', ', $terms);
      }

      $form['term_multiple'] = array(
        '#type' => 'textfield',
        '#title' => t('Taxonomy term(s)'),
        '#required' => TRUE,
        '#default_value' => isset($terms_default) ? $terms_default : '',
        '#description' => t('Enter a comma separated list of taxonomy terms to which the anchor should link.'),
      );
      $form['operator'] = array(
        '#type' => 'radios',
        '#title' => t('Operation'),
        '#options' => array(
          'and' => 'and',
          'or' => 'or',
        ),
        '#default_value' => isset($edit['operator']) ? $edit['operator'] : '',
      );
      break;
  }

  return $form;
}

/**
 * Validation callback for taxonomy link form.
 */
function autolink_ui_taxonomy_method_validate($form_values, $type) {
  switch ($type) {
    case 'select':
      if ($form_values['term_select'] == 'select') {
        form_set_error('term_select', t('You must select a term.'));
      }
      break;
    case 'multiple':
      $terms = drupal_explode_tags($form_values['term_multiple']);

      // Ensure an operation is selected if multiple terms were entered.
      if (isset($terms[1]) && empty($form_values['operator'])) {
        form_set_error('operation', t('You must select an operation for multiple term definitions.'));
      }

      // Check to ensure terms exist.
      foreach ($terms as $term) {
        if (!autolink_ui_taxonomy_term_exists($term)) {
          form_set_error('term_multiple', t('Cannot locate term data for <i>%term</i>. Please enter another term.', array('%term' => $term)));
        }
      }
      break;
  }
}

/**
 * Implements hook_autolink_LINK_TYPE_alter().
 *
 * Converts the link path to be saved to a taxonomy page. Form data is used to
 * create a proper link path linking to the results page.
 *
 * @param $link
 *   An associative array of link form data.
 *
 * @return
 *   A link array with new path which will be reference by created anchor tags.
 */
function autolink_ui_autolink_taxonomy_alter(&$link, $method) {
  switch ($method) {
    case 'term_select':
      // No extra query is needed for taxonomy so we just set the link path.
      $link['link_path'] = 'taxonomy/term/'. $link['term_select'];
      break;
    case 'term_multiple':
      $terms = drupal_explode_tags($link['term_multiple']);

      // Get term IDs for use in urls.
      foreach ($terms as $name) {
        $result = db_query("SELECT * FROM {taxonomy_term_data} WHERE name = :name", array(':name' => $name));
        foreach ($result as $term) {
          $tids[] = $term->tid;
        }
      }

      // If this is only one term then we don't need an operation.
      if (!isset($terms[1])) {
        $arg = $tids[0];
      }
      else {
        // The operation field is a special field created for filtering AND and OR statements.
        switch ($link['operator']) {
          case 'and':
            $arg = implode(',', $tids);
            break;
          case 'or':
            $arg = implode('+', $tids);
            break;
        }
      }
      // No extra query is needed for taxonomy so we just set the link path.
      $link['link_path'] = 'taxonomy/term/'. $arg;
      break;
  }
}

/**
 * Returns an array of all taxonomy terms.
 */
function _autolink_ui_taxonomy_get_terms() {
  $terms = array();
  $result = db_query('SELECT * FROM {taxonomy_term_data}');
  foreach ($result as $term) {
    $terms[$term->tid] = $term->name;
  }
  asort($terms);
  return $terms;
}

/*
 * Helper function to determine whether a term exists.
 *
 * This function is used on taxonomy link form validation to ensure that the user is
 * entering terms that exist. The terms are taken from a comma separated list of keywords
 * and validated by querying the {term_date} table.
 *
 * @param $term
 *   A term to check for.
 *
 * @return
 *   TRUE if the term exists, FALSE otherwise.
 */
function autolink_ui_taxonomy_term_exists($term) {
  return db_query("SELECT * FROM {taxonomy_term_data} WHERE name = :name", array(':name' => $term))->fetchObject;
}
